/*
  Copyright (C) 2003  Dmitry V. Levin <ldv@altlinux.org>

  Resticted shell for running "cvs server".

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#define PATH_TO_CHROOT "/path/to/chroot-cvs"
#define PATH_TO_PROGRAM "/bin/rsync"

#define _GNU_SOURCE

#include <stdio.h>
#include <errno.h>
#include <error.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <ctype.h>
#include <sys/types.h>
#include <pwd.h>

int
main (int ac, char *av[])
{
	const char *dir = PATH_TO_CHROOT;
	const char *file = 0;
	struct passwd *pw;
	uid_t   uid = getuid ();

	if (!uid)
		error (EXIT_FAILURE, 0, "must be non-root");

#ifndef WITHOUT_CHROOT_SUPPORT

	/* chroot */
	if (chroot (dir) < 0)
		error (EXIT_FAILURE, errno, "chroot");

	if (chdir ("/") < 0)
		error (EXIT_FAILURE, errno, "chdir");

	/* setcred */
	if (setuid (uid) < 0)
		error (EXIT_FAILURE, errno, "setuid");

#endif /* WITHOUT_CHROOT_SUPPORT */

	pw = getpwuid (uid);
	if (!pw)
		error (EXIT_FAILURE, errno, "getpwuid");

	if (!pw->pw_dir)
		error (EXIT_FAILURE, 0, "invalid home directory");

	if (chdir (pw->pw_dir) < 0)
		error (EXIT_FAILURE, errno, "chdir");

	/* environ */
	if (clearenv () < 0 )
		error (EXIT_FAILURE, errno, "clearenv");

	if (setenv ("USER", pw->pw_name, 1) < 0)
		error (EXIT_FAILURE, errno, "setenv: USER");

	if (setenv ("LOGNAME", pw->pw_name, 1) < 0)
		error (EXIT_FAILURE, errno, "setenv: LOGNAME");

	if (setenv ("HOME", pw->pw_dir, 1) < 0)
		error (EXIT_FAILURE, errno, "setenv: HOME");

	if (setenv ("PATH", "/bin", 1) < 0)
		error (EXIT_FAILURE, errno, "setenv: PATH");

	if (3 == ac)
	{
		if (strcmp ("-c", av[1]))
			error (EXIT_FAILURE, EINVAL, "%s", av[1]);

		if (!strcmp ("cvs server", av[2]))
		{
			const char *args[] = { "cvs", "server", 0 };

			file = PATH_TO_PROGRAM;
			execv (file, (char *const *) args);
			error (EXIT_FAILURE, errno, "execv: %s", file);
		}

		error (EXIT_FAILURE, EINVAL, "%s", av[2]);
	}

	error (EXIT_FAILURE, 0, "invalid arguments");
	return EXIT_FAILURE;
}
